@@ -23,17 +23,17 @@ Artifactory::Client - Perl client for Artifactory REST API
 
 =head1 VERSION
 
-Version 0.9.2
+Version 0.9.3
 
 =cut
 
-our $VERSION = 'v0.9.2';
+our $VERSION = 'v0.9.3';
 
 =head1 SYNOPSIS
 
 This is a Perl client for Artifactory REST API:
-https://www.jfrog.com/confluence/display/RTF/Artifactory+REST+API Every public
-method provided in this module returns a HTTP::Response object.
+https://www.jfrog.com/confluence/display/RTF/Artifactory+REST+API Every public method provided in this module returns a
+HTTP::Response object.
 
     use Artifactory::Client;
 
@@ -67,12 +67,6 @@ method provided in this module returns a HTTP::Response object.
     # Custom requests can also be made via usual get / post / put / delete requests.
     my $resp = $client->get( 'http://artifactory.server.com/path/to/resource' );
 
-Note on testing: This module is developed using Test-Driven Development.  I
-have functional tests making real API calls, however they contain proprietary
-information and I am not allowed to open source them.  The unit tests included
-are dumbed-down version of my functional tests.  They should serve as a
-detailed guide on how to make API calls.
-
 =cut
 
 has 'artifactory' => (
@@ -396,6 +390,18 @@ sub file_info {
     return $self->folder_info($path);    # should be OK to do this
 }
 
+=head2 get_storage_summary_info
+
+Returns storage summary information regarding binaries, file store and repositories
+
+=cut
+
+sub get_storage_summary_info {
+    my $self = shift;
+    my $url  = $self->_api_url() . '/storageinfo';
+    return $self->get($url);
+}
+
 =head2 item_last_modified( $path )
 
 Returns item_last_modified for a given path
@@ -446,9 +452,8 @@ sub item_properties {
 
 =head2 set_item_properties( path => $path, properties => { key => [ values ] }, recursive => 0,1 )
 
-Takes path and properties then set item properties.  Supply recursive => 0 if
-you want to suppress propagation of properties downstream.  Note that
-properties are a hashref with key-arrayref pairs, such as:
+Takes path and properties then set item properties.  Supply recursive => 0 if you want to suppress propagation of
+properties downstream.  Note that properties are a hashref with key-arrayref pairs, such as:
 
     $prop = { key1 => ['a'], key2 => ['a', 'b'] }
 
@@ -471,8 +476,8 @@ sub set_item_properties {
 
 =head2 delete_item_properties( path => $path, properties => [ key_names ], recursive => 0,1 )
 
-Takes path and properties then delete item properties.  Supply recursive => 0
-if you want to suppress propagation of properties downstream.
+Takes path and properties then delete item properties.  Supply recursive => 0 if you want to suppress propagation of
+properties downstream.
 
 =cut
 
@@ -489,10 +494,27 @@ sub delete_item_properties {
     return $self->delete($url);
 }
 
+=head2 set_item_sha256_checksum( repoKey => 'foo', path => 'bar' )
+
+Calculates an artifact's SHA256 checksum and attaches it as a property (with key "sha256"). If the artifact is a folder,
+then recursively calculates the SHA256 of each item in the folder and attaches the property to each item.
+
+=cut
+
+sub set_item_sha256_checksum {
+    my ( $self, %args ) = @_;
+    my $url = $self->_api_url() . '/checksum/sha256';
+    return $self->put(
+        $url,
+        "Content-Type" => 'application/json',
+        Content        => $self->_json->encode( \%args )
+    );
+}
+
 =head2 retrieve_artifact( $path, $filename )
 
-Takes path and retrieves artifact on the path.  If $filename is given, artifact
-content goes into the $filename rather than the HTTP::Response object.
+Takes path and retrieves artifact on the path.  If $filename is given, artifact content goes into the $filename rather
+than the HTTP::Response object.
 
 =cut
 
@@ -564,8 +586,7 @@ sub trace_artifact_retrieval {
 
 =head2 archive_entry_download( $path, $archive_path )
 
-Takes path and archive_path, retrieves an archived resource from the specified
-archive destination.
+Takes path and archive_path, retrieves an archived resource from the specified archive destination.
 
 =cut
 
@@ -578,8 +599,7 @@ sub archive_entry_download {
 
 =head2 create_directory( path => $path, properties => { key => [ values ] } )
 
-Takes path, properties then create a directory.  Directory needs to end with a
-/, such as "/some_dir/".
+Takes path, properties then create a directory.  Directory needs to end with a /, such as "/some_dir/".
 
 =cut
 
@@ -590,8 +610,8 @@ sub create_directory {
 
 =head2 deploy_artifact( path => $path, properties => { key => [ values ] }, file => $file )
 
-Takes path on Artifactory, properties and filename then deploys the file.  Note
-that properties are a hashref with key-arrayref pairs, such as:
+Takes path on Artifactory, properties and filename then deploys the file.  Note that properties are a hashref with
+key-arrayref pairs, such as:
 
     $prop = { key1 => ['a'], key2 => ['a', 'b'] }
 
@@ -613,16 +633,16 @@ sub deploy_artifact {
     my $url = join( ";", @joiners );
     my $req = HTTP::Request::StreamingUpload->new(
         PUT     => $url,
-        path    => $file,
         headers => HTTP::Headers->new( %{$header} ),
+        ( $file ? ( fh => Path::Tiny::path($file)->openr_raw() ) : () ),
     );
     return $self->request($req);
 }
 
 =head2 deploy_artifact_by_checksum( path => $path, properties => { key => [ values ] }, file => $file, sha1 => $sha1 )
 
-Takes path, properties, filename and sha1 then deploys the file.  Note that
-properties are a hashref with key-arrayref pairs, such as:
+Takes path, properties, filename and sha1 then deploys the file.  Note that properties are a hashref with key-arrayref
+pairs, such as:
 
     $prop = { key1 => ['a'], key2 => ['a', 'b'] }
 
@@ -642,8 +662,7 @@ sub deploy_artifact_by_checksum {
 
 =head2 deploy_artifacts_from_archive( path => $path, file => $file )
 
-Path is the path on Artifactory, file is path to local archive.  Will deploy
-$file to $path.
+Path is the path on Artifactory, file is path to local archive.  Will deploy $file to $path.
 
 =cut
 
@@ -674,8 +693,8 @@ sub push_a_set_of_artifacts_to_bintray {
 
 =head2 push_docker_tag_to_bintray( dockerImage => 'jfrog/ubuntu:latest', async => 'true', ... )
 
-Push Docker tag to Bintray.  Calculation can be synchronous (the default) or asynchronous.
-You will need to enter your Bintray credentials, for more details, please refer to Entering your Bintray credentials.
+Push Docker tag to Bintray.  Calculation can be synchronous (the default) or asynchronous.  You will need to enter your
+Bintray credentials, for more details, please refer to Entering your Bintray credentials.
 
 =cut
 
@@ -718,9 +737,8 @@ sub delete_item {
 
 =head2 copy_item( from => $from, to => $to, dry => 1, suppressLayouts => 0/1, failFast => 0/1 )
 
-Copies an artifact from $from to $to.  Note that for this particular API call,
-the $from and $to must include repository names as copy source and destination
-may be different repositories.  You can also supply dry, suppressLayouts and
+Copies an artifact from $from to $to.  Note that for this particular API call, the $from and $to must include repository
+names as copy source and destination may be different repositories.  You can also supply dry, suppressLayouts and
 failFast values as specified in the documentation.
 
 =cut
@@ -733,9 +751,8 @@ sub copy_item {
 
 =head2 move_item( from => $from, to => $to, dry => 1, suppressLayouts => 0/1, failFast => 0/1 )
 
-Moves an artifact from $from to $to.  Note that for this particular API call,
-the $from and $to must include repository names as copy source and destination
-may be different repositories.  You can also supply dry, suppressLayouts and
+Moves an artifact from $from to $to.  Note that for this particular API call, the $from and $to must include repository
+names as copy source and destination may be different repositories.  You can also supply dry, suppressLayouts and
 failFast values as specified in the documentation.
 
 =cut
@@ -859,8 +876,7 @@ sub delete_local_multi_push_replication {
 
 =head2 file_list( $dir, %opts )
 
-Get a flat (the default) or deep listing of the files and folders (not included
-by default) within a folder
+Get a flat (the default) or deep listing of the files and folders (not included by default) within a folder
 
 =cut
 
@@ -1035,8 +1051,7 @@ sub license_search {
 
 =head2 artifact_version_search( g => 'foo', a => 'bar', v => '1.0', repos => [ 'foo', 'bar' ] )
 
-Search for all available artifact versions by GroupId and ArtifactId in local,
-remote or virtual repositories
+Search for all available artifact versions by GroupId and ArtifactId in local, remote or virtual repositories
 
 =cut
 
@@ -1047,8 +1062,7 @@ sub artifact_version_search {
 
 =head2 artifact_latest_version_search_based_on_layout( g => 'foo', a => 'bar', v => '1.0', repos => [ 'foo', 'bar' ] )
 
-Search for the latest artifact version by groupId and artifactId, based on the
-layout defined in the repository
+Search for the latest artifact version by groupId and artifactId, based on the layout defined in the repository
 
 =cut
 
@@ -1245,8 +1259,7 @@ sub get_permission_target_details {
 
 =head2 create_or_replace_permission_target( $name, %args )
 
-Creates a new permission target in Artifactory or replaces an existing
-permission target
+Creates a new permission target in Artifactory or replaces an existing permission target
 
 =cut
 
@@ -1369,8 +1382,7 @@ sub set_gpg_pass_phrase {
 
 =head2 get_repositories( $type )
 
-Returns a list of minimal repository details for all repositories of the
-specified type
+Returns a list of minimal repository details for all repositories of the specified type
 
 =cut
 
@@ -1405,8 +1417,8 @@ sub repository_configuration {
 
 =head2 create_or_replace_repository_configuration( $name, \%payload, %args )
 
-Creates a new repository in Artifactory with the provided configuration or
-replaces the configuration of an existing repository
+Creates a new repository in Artifactory with the provided configuration or replaces the configuration of an existing
+repository
 
 =cut
 
@@ -1417,8 +1429,7 @@ sub create_or_replace_repository_configuration {
 
 =head2 update_repository_configuration( $name, \%payload )
 
-Updates an exiting repository configuration in Artifactory with the provided
-configuration elements
+Updates an exiting repository configuration in Artifactory with the provided configuration elements
 
 =cut
 
@@ -1440,8 +1451,7 @@ sub delete_repository {
 
 =head2 calculate_yum_repository_metadata( async => 0/1 )
 
-Calculates/recalculates the YUM metdata for this repository, based on the RPM
-package currently hosted in the repository
+Calculates/recalculates the YUM metdata for this repository, based on the RPM package currently hosted in the repository
 
 =cut
 
@@ -1453,9 +1463,8 @@ sub calculate_yum_repository_metadata {
 
 =head2 calculate_nuget_repository_metadata
 
-Recalculates all the NuGet packages for this repository (local/cache/virtual),
-and re-annotate the NuGet properties for each NuGet package according to it's
-internal nuspec file
+Recalculates all the NuGet packages for this repository (local/cache/virtual), and re-annotate the NuGet properties for
+each NuGet package according to it's internal nuspec file
 
 =cut
 
@@ -1634,8 +1643,7 @@ sub install_license {
 
 =head2 version_and_addons_information
 
-Retrieve information about the current Artifactory version, revision, and
-currently installed Add-ons
+Retrieve information about the current Artifactory version, revision, and currently installed Add-ons
 
 =cut
 
@@ -1671,8 +1679,7 @@ sub execute_plugin_code {
 
 =head2 retrieve_all_available_plugin_info
 
-Retrieves all available user plugin information (subject to the permissions of
-the provided credentials)
+Retrieves all available user plugin information (subject to the permissions of the provided credentials)
 
 =cut
 
@@ -1683,8 +1690,8 @@ sub retrieve_all_available_plugin_info {
 
 =head2 retrieve_plugin_info_of_a_certain_type( $type )
 
-Retrieves all available user plugin information (subject to the permissions of
-the provided credentials) of the specified type
+Retrieves all available user plugin information (subject to the permissions of the provided credentials) of the
+specified type
 
 =cut
 
@@ -1704,15 +1711,14 @@ sub retrieve_build_staging_strategy {
     my $strategy_name = delete $args{strategyName};
     my $build_name    = delete $args{buildName};
 
-    my $url = $self->_api_url() . "/plugins/build/staging/$strategy_name?buildName=$build_name?params=";
+    my $url = $self->_api_url() . "/plugins/build/staging/$strategy_name?buildName=$build_name&params=";
     $url = $url . $self->_attach_properties( properties => \%args );
     return $self->get($url);
 }
 
 =head2 execute_build_promotion( promotionName => 'promotion1', buildName => 'build1', buildNumber => 3, %args )
 
-Executes a named promotion closure found in the promotions section of a user
-plugin
+Executes a named promotion closure found in the promotions section of a user plugin
 
 =cut
 
@@ -1727,6 +1733,19 @@ sub execute_build_promotion {
     return $self->post($url);
 }
 
+=head2 reload_plugins
+
+Reloads user plugins if there are modifications since the last user plugins reload. Works regardless of the automatic
+user plugins refresh interval
+
+=cut
+
+sub reload_plugins {
+    my $self = shift;
+    my $url  = $self->_api_url() . '/plugins/reload';
+    return $self->post($url);
+}
+
 =head1 IMPORT & EXPORT
 
 =cut
